from iutest.qt import QtGui

LOG_COLOR_INFORMATION = QtGui.QColor(200, 200, 200)
LOG_COLOR_ERROR = QtGui.QColor(234, 52, 95)
LOG_COLOR_FAILED = QtGui.QColor(234, 52, 95)
LOG_COLOR_WARNING = QtGui.QColor(220, 206, 135)
LOG_COLOR_SUCCESS = QtGui.QColor(138, 211, 11)
